@@ -2,25 +2,31 @@
 //!
 //! A [Gaussian integer] is a complex number whose real and imaginary parts are both integers.
 //!
-//! The centerpiece of this crate is a method to test for a [Gaussian prime].
-//!
 //! [Gaussian integer]: https://en.wikipedia.org/wiki/Gaussian_integer
 //! [Gaussian prime]: crate::GaussianInt#method.is_gaussian_prime
 #![deny(missing_docs)]
 #![allow(clippy::needless_return)]
 
+#[cfg(doctest)]
+#[macro_use]
+extern crate doc_comment;
+
+#[cfg(doctest)]
+doctest!("../README.md", readme);
+
 use std::fmt;
 
 use num_complex::Complex;
-use num_traits::{PrimInt, Signed};
+use num_integer::Integer;
+use num_traits::{One, PrimInt, Signed, Zero};
 
 mod ops;
 
-/// A Gaussian integer.
+/// A [Gaussian integer] is a complex number whose real and imaginary parts are both integers.
 ///
-/// This is a complex number whose real and imaginary parts are both integers.
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
-pub struct GaussianInt<T: PrimInt>(pub Complex<T>);
+/// [Gaussian integer]: https://en.wikipedia.org/wiki/Gaussian_integer
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
+pub struct GaussianInt<T: PrimInt + Integer>(pub Complex<T>);
 
 /// Creates a new [`GaussianInt`].
 ///
@@ -36,34 +42,33 @@ pub struct GaussianInt<T: PrimInt>(pub Complex<T>);
 /// ```
 #[macro_export]
 macro_rules! gaussint {
-    ( $a:expr, $b:expr ) => {
+    ($a:expr,$b:expr) => {
         GaussianInt::new($a, $b)
     };
+    ($n:expr) => {
+        GaussianInt::new($n, 0)
+    };
 }
 
-impl<T: PrimInt> GaussianInt<T> {
+impl<T: PrimInt + Integer> GaussianInt<T> {
     #[allow(missing_docs)]
     pub fn new(r: T, i: T) -> Self {
         Self(Complex::new(r, i))
     }
 
-    /// Returns zero as a [`GaussianInt`].
-    pub fn zero() -> Self {
-        Self(Complex::new(T::zero(), T::zero()))
-    }
-
-    /// Given a Gaussian integer z0, called a modulus,
-    /// two Gaussian integers z1, z2 are *congruent modulo z0*,
-    /// if their difference is a multiple of z0.
+    /// Given a Gaussian integer z₀, called a *modulus*,
+    /// two Gaussian integers z₁, z₂ are *congruent modulo z₀*,
+    /// if their difference is a multiple of z₀.
     ///
     /// # Example
     ///
     /// ```
-    /// # use gaussiant::GaussianInt;
+    /// # use gaussiant::{GaussianInt, gaussint};
     /// # fn main() {
-    /// let c1 = GaussianInt::new(5, 0);
-    /// let c2 = GaussianInt::new(25, 0);
-    /// let c3 = GaussianInt::new(10, 0);
+    /// // 2 + 5i ≡ i mod 1 + 2i
+    /// let c1 = gaussint!(2, 5);
+    /// let c2 = gaussint!(0, 1);
+    /// let c3 = gaussint!(1, 2);
     /// assert!(c1.congruent(c2, c3));
     /// # }
     /// ```
@@ -72,7 +77,7 @@ impl<T: PrimInt> GaussianInt<T> {
     }
 }
 
-impl<T: PrimInt + Signed> GaussianInt<T> {
+impl<T: PrimInt + Integer + Signed> GaussianInt<T> {
     /// Returns the complex conjugate.
     ///
     /// # Example
@@ -103,7 +108,7 @@ impl<T: PrimInt + Signed> GaussianInt<T> {
         *self * self.conj()
     }
 
-    /// Returns `true` if `self` is a divisor of `other`.
+    /// Returns `true` if `self` divides `other`.
     ///
     /// # Example
     ///
@@ -112,11 +117,11 @@ impl<T: PrimInt + Signed> GaussianInt<T> {
     /// # fn main() {
     /// let c1 = GaussianInt::new(5, 0);
     /// let c2 = GaussianInt::new(1, 2);
-    /// assert!(c2.is_divisor_of(c1));
+    /// assert!(c2.divides(c1));
     /// # }
     /// ```
-    pub fn is_divisor_of(&self, other: Self) -> bool {
-        (other % *self) == Self::zero() && !(other.0.re != T::zero() && other.0.im != T::zero())
+    pub fn divides(&self, other: Self) -> bool {
+        *self != Self::zero() && (other % *self) == Self::zero()
     }
 
     /// Tests whether a Gaussian integer is a rational integer.
@@ -162,37 +167,80 @@ impl<T: PrimInt + Signed> GaussianInt<T> {
         let a = self.0.re;
         let b = self.0.im;
 
+        // These numbers would cause integer overflow panics below.
+        match (a.abs().to_isize().unwrap(), b.abs().to_isize().unwrap()) {
+            (0, 0) => return false,
+            (1, 1) => return true,
+            (-1, -1) => return true,
+            (2, 0) => return false,
+            (0, 2) => return false,
+            _ => {}
+        }
+
         let condition_1 = match (a.is_zero(), b.is_zero()) {
             (true, false) => {
-                let other = b.to_isize().unwrap();
-                is_prime(other) && (other - 3) % 4 == 0
+                let other = b.abs().to_u64().unwrap();
+                primal::is_prime(other) && (other - 3) % 4 == 0
             }
             (false, true) => {
-                let other = a.to_isize().unwrap();
-                is_prime(other) && (other - 3) % 4 == 0
+                let other = a.abs().to_u64().unwrap();
+                primal::is_prime(other) && (other - 3) % 4 == 0
             }
             _ => false,
         };
 
         let condition_2 = match (a.is_zero(), b.is_zero()) {
             (false, false) => {
-                let a = a.to_isize().unwrap();
-                let b = b.to_isize().unwrap();
-                let both_prime = is_prime(a) && is_prime(b);
-                let sum_of_squares = isize::pow(a, 2) + isize::pow(b, 2);
+                let a = a.abs().to_u64().unwrap();
+                let b = b.abs().to_u64().unwrap();
+                let sum_of_squares = u64::pow(a, 2) + u64::pow(b, 2);
                 let sum_of_squares_is_4n_plus_3 = (sum_of_squares - 3) % 4 == 0;
-                both_prime && is_prime(sum_of_squares) && !sum_of_squares_is_4n_plus_3
+                primal::is_prime(sum_of_squares) && !sum_of_squares_is_4n_plus_3
             }
             _ => false,
         };
 
         condition_1 || condition_2
     }
-}
 
-impl GaussianInt<isize> {
+    /// Returns an array of the units of ℤ\[*i*\], the ring of Gaussian integers.
+    ///
+    /// The units are 1, -1, *i*, -*i*.
+    pub fn units() -> [Self; 4] {
+        [
+            Self::one(),                     //  1
+            -Self::one(),                    // -1
+            Self::new(T::zero(), T::one()),  //  i
+            Self::new(T::zero(), -T::one()), // -i
+        ]
+    }
+
+    /// Gaussian integers are called associates
+    /// if they can be obtained from one another by multiplication by units.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # use gaussiant::GaussianInt;
+    /// # fn main() {
+    /// let z1 = GaussianInt::new(1, 1);
+    /// let z2 = GaussianInt::new(1, -1);
+    /// assert!(z1.is_associated(z2));
+    /// # }
+    /// ```
+    pub fn is_associated(&self, other: Self) -> bool {
+        for u in GaussianInt::units() {
+            if (*self * u) == other {
+                return true;
+            }
+        }
+        false
+    }
+
     /// Tests whether a Gaussian integer is "even."
     ///
+    /// A Gaussian integer *z* is "even" if *z* ≡ 0 mod 1+*i*.
+    ///
     /// See <https://en.wikipedia.org/wiki/Gaussian_integer#Examples>.
     ///
     /// # Example
@@ -205,11 +253,15 @@ impl GaussianInt<isize> {
     /// # }
     /// ```
     pub fn is_even(&self) -> bool {
-        self.congruent(Self::zero(), Self::new(1, 1))
+        let modulus = Self::new(T::one(), T::one());
+        // self ≡ 0 mod 1+i
+        self.congruent(Self::zero(), modulus)
     }
 
     /// Tests whether a Gaussian integer is "odd."
     ///
+    /// A Gaussian integer *z* is "odd" if *z* ≡ 1 mod 1+*i*.
+    ///
     /// See <https://en.wikipedia.org/wiki/Gaussian_integer#Examples>.
     ///
     /// # Example
@@ -222,13 +274,13 @@ impl GaussianInt<isize> {
     /// # }
     /// ```
     pub fn is_odd(&self) -> bool {
-        let one = Self::new(1, 0);
-        self.congruent(one, Self::new(1, 1))
-        // *self % (Self::new(1, 1) + one) == Self::zero()
+        let modulus = Self::new(T::one(), T::one());
+        // self ≡ 1 mod 1+i
+        self.congruent(Self::one(), modulus)
     }
 }
 
-impl<T: PrimInt + Signed> GaussianInt<T>
+impl<T: PrimInt + Integer + Signed> GaussianInt<T>
 where
     f64: From<T>,
 {
@@ -254,12 +306,12 @@ where
     }
 }
 
-/// Returns an iterator of all Gaussian primes with positive real parts
-/// and with integer parts below *n*.
-pub fn get_positive_primes(n: isize) -> impl Iterator<Item = GaussianInt<isize>> + 'static {
+/// Returns an iterator of all Gaussian primes *a* + *b*i
+/// where |a|,|b| ≤ `n`.
+pub fn get_g_primes(n: isize) -> impl Iterator<Item = GaussianInt<isize>> + 'static {
     let mut primes: Vec<GaussianInt<_>> = vec![];
-    for a in 0..=n {
-        for b in 0..=n {
+    for a in -n..=n {
+        for b in -n..=n {
             let z = GaussianInt::new(a, b);
             if z.is_gaussian_prime() {
                 primes.push(z);
@@ -269,11 +321,11 @@ pub fn get_positive_primes(n: isize) -> impl Iterator<Item = GaussianInt<isize>>
     primes.into_iter()
 }
 
-/// Returns an iterator of all Gaussian integers with positive real parts
-/// and with integer parts below *n*.
+/// Returns an iterator of all Gaussian integers *a* + *b*i
+/// where |*a*|,|*b*| ≤ `n`.
 pub fn get_g_ints(n: isize) -> impl Iterator<Item = GaussianInt<isize>> + 'static {
     let mut primes: Vec<GaussianInt<_>> = vec![];
-    for a in 0..=n {
+    for a in -n..=n {
         for b in -n..=n {
             let z = GaussianInt::new(a, b);
             primes.push(z);
@@ -282,28 +334,35 @@ pub fn get_g_ints(n: isize) -> impl Iterator<Item = GaussianInt<isize>> + 'stati
     primes.into_iter()
 }
 
-impl<T: PrimInt> From<Complex<T>> for GaussianInt<T> {
-    fn from(g: Complex<T>) -> Self {
-        Self(g)
+impl<T: PrimInt + Integer> One for GaussianInt<T> {
+    fn one() -> Self {
+        GaussianInt::new(T::one(), T::zero())
     }
 }
 
-impl<T: PrimInt> From<GaussianInt<T>> for isize {
+impl<T: PrimInt + Integer> Zero for GaussianInt<T> {
+    fn zero() -> Self {
+        GaussianInt::new(T::zero(), T::zero())
+    }
+
+    fn is_zero(&self) -> bool {
+        *self == GaussianInt::zero()
+    }
+}
+
+impl<T: PrimInt + Integer> From<Complex<T>> for GaussianInt<T> {
+    fn from(z: Complex<T>) -> Self {
+        Self(z)
+    }
+}
+
+impl<T: PrimInt + Integer> From<GaussianInt<T>> for isize {
     fn from(g: GaussianInt<T>) -> Self {
         g.0.re.to_isize().unwrap()
     }
 }
 
-fn is_prime(number: isize) -> bool {
-    for i in 2..(number / 2 + 1) {
-        if number % i == 0 {
-            return false;
-        }
-    }
-    return number > 1;
-}
-
-impl<T: PrimInt> fmt::Display for GaussianInt<T> {
+impl<T: PrimInt + Integer> fmt::Display for GaussianInt<T> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         let zero = T::zero();
         if self.0.im < zero {
